'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var t = require('assert');
var http = require('http');

var request = require('../../').client;

describe('redirect', function () {
  var server;

  before(function (done) {
    var counter = 0;
    server = http.createServer();
    server.on('request', function (req, res) {
      var _req$url$split = req.url.split('?'),
          _req$url$split2 = _slicedToArray(_req$url$split, 2),
          _ = _req$url$split2[0],
          query = _req$url$split2[1];

      if (/^\/absolute/.test(req.url)) {
        res.writeHead(301, { location: 'http://localhost:5000/target' });
        res.end(query || 'not ok');
      } else if (/^\/relative/.test(req.url)) {
        res.writeHead(301, { location: '/target' });
        res.end(query || 'not ok');
      } else if (/^\/target/.test(req.url)) {
        res.writeHead(200, query ? { 'content-type': 'application/x-www-form-urlencoded' } : {});
        res.end(query || 'ok');
      } else if (req.url === '/stuck') {
        res.writeHead(301, { location: 'http://localhost:5000/stuck' });
        res.end((++counter).toString());
      }
    });
    server.listen(5000, done);
  });

  it('do not follow patch|put|post|delete redirects', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var _ref2, res, body;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return request({
              method: 'POST',
              url: 'http://localhost:5000/absolute'
            });

          case 2:
            _ref2 = _context.sent;
            res = _ref2.res;
            body = _ref2.body;

            t.strictEqual(res.statusCode, 301);
            t.equal(res.statusMessage, 'Moved Permanently');
            t.equal(body, 'not ok');

          case 8:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('absolute URL', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2() {
    var _ref4, body;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return request({
              url: 'http://localhost:5000/absolute'
            });

          case 2:
            _ref4 = _context2.sent;
            body = _ref4.body;

            t.equal(body, 'ok', 'should follow absolute URLs');

          case 5:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('absolute URL + querystring', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var _ref6, body;

    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return request({
              url: 'http://localhost:5000/absolute',
              qs: { a: 1 }
            });

          case 2:
            _ref6 = _context3.sent;
            body = _ref6.body;

            t.deepEqual(body, { a: '1' }, 'should persist URL querystring');

          case 5:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));

  it('relative URL', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4() {
    var _ref8, body;

    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return request({
              url: 'http://localhost:5000/relative'
            });

          case 2:
            _ref8 = _context4.sent;
            body = _ref8.body;

            t.equal(body, 'ok', 'should follow relative URLs');

          case 5:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, undefined);
  })));

  it('relative URL + querystring', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5() {
    var _ref10, body;

    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return request({
              url: 'http://localhost:5000/relative',
              qs: { a: 1 }
            });

          case 2:
            _ref10 = _context5.sent;
            body = _ref10.body;

            t.deepEqual(body, { a: '1' }, 'should persist URL querystring');

          case 5:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, undefined);
  })));

  it('stuck in redirect loop', _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6() {
    var args;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            args = {
              url: 'http://localhost:5000/stuck'
            };
            _context6.prev = 1;
            _context6.next = 4;
            return request(args);

          case 4:
            _context6.next = 11;
            break;

          case 6:
            _context6.prev = 6;
            _context6.t0 = _context6['catch'](1);

            t.equal(_context6.t0.message, 'request-compose: exceeded maximum redirects', 'throw error on maximum redirects exceeded');
            t.equal(_context6.t0.body, '3', 'by default maximum of 3 redirects are allowed');
            t.deepEqual(args, { url: 'http://localhost:5000/stuck' }, 'input args should not be modified');

          case 11:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, undefined, [[1, 6]]);
  })));

  after(function (done) {
    server.close(done);
  });
});